<?php

namespace Quivers\Integration\Observer;

use Quivers\Integration\Block\Adminhtml\Form\Button;
use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Event\Observer as EventObserver;
use Psr\Log\LoggerInterface as Logger;
use Magento\Framework\Exception\CouldNotSaveException;

class ConfigObserver implements ObserverInterface
{
    /**
     * @var Logger
     */
    protected $logger;
    protected $curlClient;
    protected $scopeConfig;
    protected $configWriter;
    protected $configValueFactory;
    protected $_responseFactory;
    public $layout;
    public $_backendUrl;
    protected $_productCollectionFactory;

    /**
     * @param Logger $logger
     */
    
    const XML_MARKETPLACE_ID = 'quivers/parameters/marketplaceId';
    const XML_PRODUCTGROUP_ID = 'quivers/parameters/ProductGroupId';
    const XML_QUIVERS_API_KEY = 'quivers/parameters/quivers_api_key';
    const XML_QUIVERS_BUSINESS_REFID = 'quivers/parameters/merchant_ref_id';
    const XML_MAGENTO_ACCESS_TOKEN = 'quivers/parameters/magento_access_token';
    const XML_MAGENTO_BASE_URL = 'quivers/parameters/magento_base_url';
    const XML_QUIVERS_SANDBOX = 'quivers/parameters/quivers_sanndbox';
    const XML_USD_MARKETPLACE_ID = 'quivers_tax/marketplaces/usdMarketPlaceId';
    const XML_EURO_MARKETPLACE_ID = 'quivers_tax/marketplaces/euroMarketPlaceId';
    const XML_CAD_MARKETPLACE_ID = 'quivers_tax/marketplaces/cadMarketPlaceId';
    const XML_AUD_MARKETPLACE_ID ='quivers_tax/marketplaces/audMarketPlaceId';
    const XML_JPY_MARKETPLACE_ID = 'quivers_tax/marketplaces/jpyMarketPlaceId';
    const XML_GBP_MARKETPLACE_ID = 'quivers_tax/marketplaces/gbpMarketPlaceId';
    const XML_QUIVERS_UUID = 'quivers/parameters/quivers_uuid';
    const XML_ENABLED = 'quivers/parameters/active';
    const XML_QUIVERS_UPC = 'quivers/parameters/quivers_custom_upc';

    protected $baseUrl = 'https://middleware.quiversdemo.com/profile';


    public function __construct(
        Logger $logger,
        \Magento\Framework\HTTP\Client\Curl $curl,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Framework\App\Config\Storage\WriterInterface $configWriter,
        \Magento\Framework\App\Config\ValueFactory $configValueFactory,
        \Magento\Framework\View\LayoutInterface $layout,
        \Magento\Backend\Model\Session $backendSession,
        \Magento\Catalog\Model\Session $catalogSession,
        \Magento\Backend\Model\UrlInterface $backendUrl,
        \Magento\Framework\App\ResponseFactory $responseFactory,
        \Magento\Framework\App\Request\Http $request,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Catalog\Model\ResourceModel\Product\CollectionFactory $productCollectionFactory
    ) {
        $this->curlClient = $curl;
        $this->logger = $logger;
        $this->scopeConfig = $scopeConfig;
        $this->configWriter = $configWriter;
        $this->configValueFactory = $configValueFactory;
        $this->layout = $layout;
        $this->backendSession = $backendSession;
        $this->_catalogSession = $catalogSession;
        $this->_backendUrl = $backendUrl;
        $this->_responseFactory = $responseFactory;
        $this->request = $request;
        $this->storeManager = $storeManager;
        $this->_productCollectionFactory = $productCollectionFactory;
    }

    public function execute(EventObserver $observer)
    {
        $storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
        $marketPlaceId = $this->scopeConfig->getValue(self::XML_MARKETPLACE_ID, $storeScope);
        $usdMarketPlaceId = $this->scopeConfig->getValue(self::XML_USD_MARKETPLACE_ID, $storeScope);
        $euroMarketPlaceId = $this->scopeConfig->getValue(self::XML_EURO_MARKETPLACE_ID, $storeScope);
        $cadMarketPlaceId = $this->scopeConfig->getValue(self::XML_CAD_MARKETPLACE_ID, $storeScope);
        $audMarketPlaceId = $this->scopeConfig->getValue(self::XML_AUD_MARKETPLACE_ID, $storeScope);
        $jpyMarketPlaceId = $this->scopeConfig->getValue(self::XML_JPY_MARKETPLACE_ID, $storeScope);
        $gbpMarketPlaceId = $this->scopeConfig->getValue(self::XML_GBP_MARKETPLACE_ID, $storeScope);
        $productGroupId = $this->scopeConfig->getValue(self::XML_PRODUCTGROUP_ID, $storeScope);
        $quiversAPIKey = $this->scopeConfig->getValue(self::XML_QUIVERS_API_KEY, $storeScope);
        $magentoBaseUrl = $this->scopeConfig->getValue(self::XML_MAGENTO_BASE_URL, $storeScope);
        $quiversSandbox = $this->scopeConfig->getValue(self::XML_QUIVERS_SANDBOX, $storeScope);
        $quiversBusinessRefId = $this->scopeConfig->getValue(self::XML_QUIVERS_BUSINESS_REFID, $storeScope);
        $magentoAcessToken = $this->scopeConfig->getValue(self::XML_MAGENTO_ACCESS_TOKEN, $storeScope);
        $storeId = (int) $this->request->getParam('store',0);
        $active = $this->scopeConfig->getValue(self::XML_ENABLED, $storeScope, $storeId);
        $quivers_upc = $this->scopeConfig->getValue(self::XML_QUIVERS_UPC, $storeScope);
        // $is_active = "false"; if($active){$is_active = "true";}
        if($quiversSandbox == 0) {
            $this->baseUrl = "https://middleware.quivers.com/profile";
        }
        $blockObj = $this->layout->createBlock('Quivers\Integration\Block\Adminhtml\Form\Button');
        if (filter_var($magentoBaseUrl, FILTER_VALIDATE_URL) === FALSE) {
            $errorMessage="Invalid Magento Base Url. Please try again. If the issue still persists, please contact enterprise@quivers.com for further assistance.";
            $blockObj->getBackendSession()->unsMyValue();
            $blockObj->getBackendSession()->setMyName('InActive');
            $blockObj->getCatalogSession()->setMyName('quivers_parameters_magento_base_url');
            throw new CouldNotSaveException(
                __($errorMessage)); 
        }
        
        $this->validate_magento_creds($magentoBaseUrl,$magentoAcessToken,$blockObj,$quiversSandbox);
        $profileUrl = $this->baseUrl . "/create";
        $profileUpdateUrl = $this->baseUrl . "/update/marketplaces";

        $product_group_id_array = explode(", ", $productGroupId);
        $product_group_id_list = [];
        foreach ($product_group_id_array as $productGroupId) {
            array_push($product_group_id_list, $productGroupId);
        }
        $payload = [
            "client_type" => "Magento2",
            "base_url" => $magentoBaseUrl,
            "business_refid" => $quiversBusinessRefId,
            "api_key" => $quiversAPIKey,
            "auth_token" => "Bearer ".$magentoAcessToken,
            "enable_plugin" => "true"
        ];
        $this->curlClient->addHeader("Content-Type", "application/json");
        $this->curlClient->post($profileUrl, json_encode($payload));
        $id=[ 'api' => 'quivers_parameters_quivers_api_key',
                 'ref_id' => 'quivers_parameters_merchant_ref_id'];
        $response = $this->curlClient->getBody();
        $blockObj->getBackendSession()->unsMyValue();
        $blockObj->getBackendSession()->setMyName('InActive');
        $collection = $this->_productCollectionFactory->create();
        $collection->addAttributeToSelect('*');
        foreach ($collection as $product) 
        {
            $product->setData('backend_upc',"quivers_custom_upc_".$quivers_upc);
		    $product->getResource()->saveAttribute($product,'backend_upc');
        } 
        if($this->curlClient->getStatus()==200){
            $uuid = json_decode($response)->uuid;
            if($uuid) {
                $save = $this->configWriter->save(self::XML_QUIVERS_UUID, $uuid);
            }
            if (json_decode($response)->isactive=='true'){
                $blockObj->getBackendSession()->unsMyValue();
                $blockObj->getBackendSession()->setMyName('Active');
            }
            $url = $this->_backendUrl->getUrl("adminhtml/system_config/edit/section/quivers_tax");
            $_objectManager = \Magento\Framework\App\ObjectManager::getInstance();
            $this->_responseFactory->create()->setRedirect($url)->sendResponse();
            die();
        }
        $allMarketPlaces = [];

        $marketPlaceIds = [
            "USD" => $usdMarketPlaceId,
            "CAD" => $cadMarketPlaceId,
            "EURO" => $euroMarketPlaceId,
            "AUD" => $audMarketPlaceId,
            "JPY" => $jpyMarketPlaceId,
            "GBP" => $gbpMarketPlaceId
        ];
        foreach($marketPlaceIds as $currency=>$mPlaceId) {
            if($mPlaceId) {
                $allMarketPlaces[] = [
                    'marketplace_id' => $mPlaceId,
                    'currency' => $currency,
                    'claiming_group_ids' => $product_group_id_list         
                ];
            }
        }
        if($this->curlClient->getStatus()==200) {
            $response = json_decode($response, true);
            $uuid = $response["uuid"];
            if($uuid) {
                $save = $this->configWriter->save(self::XML_QUIVERS_UUID, $uuid);
            }
            if(count($allMarketPlaces)>0) {
                $payload = [ "marketplaces" => $allMarketPlaces ];
                $this->curlClient->addHeader("uuid", $uuid);
                $this->curlClient->post($profileUpdateUrl, json_encode($payload));
                $response = $this->curlClient->getBody();
                if($this->curlClient->getStatus()>=400) {
                    $errorMessage = 'An error occurred while updating the profile. ERROR: '.$response;
                    throw new CouldNotSaveException(
                        __($errorMessage));
                }
            }
        } else {
            if($this->curlClient->getStatus()>=500){
            $errorMessage = json_decode($response)->message;
            }
            else{
            try{
            $errorMessage=json_decode($response)->error->message;
            $key=json_decode($response)->error->key;
            foreach($id as $keys=>$fieldId) {
            if($keys==$key) {
               $blockObj->getCatalogSession()->setMyName($fieldId); 
            }}}
            catch (\Exception $e){
                $errorMessage="Invalid Form Details Please try again. If the issue still persists, please contact enterprise@quivers.com for further assistance.";
            }
            throw new CouldNotSaveException(
                __($errorMessage));
        }}

    }
    
    public function validate_magento_creds($magentoBaseUrl,$magentoAcessToken,$blockObj,$quiversSandbox){
        if ($quiversSandbox == 0 and parse_url($magentoBaseUrl)['scheme'] != 'https')
        {
            $errorMessage='Invalid Magento Base Url. Please ensure that if test mode is disabled , set the base URL to HTTPS. If the issue still persists, please contact enterprise@quivers.com for further assistance.';
            $blockObj->getCatalogSession()->setMyName('quivers_parameters_magento_base_url');
            $blockObj->getBackendSession()->unsMyValue();
            $blockObj->getBackendSession()->setMyName('InActive');
        throw new CouldNotSaveException(
                __($errorMessage));
        }
        $endpoint = '/V1/orders';
        $url = $magentoBaseUrl.$endpoint;
        $date=date("Y-m-d H:i:s");
        $params = array('searchCriteria[filter_groups][0][filters][0][field]'=>'status',
            'searchCriteria[filter_groups][0][filters][0][value]'=> 'pending',
            'searchCriteria[filter_groups][0][filters][0][condition_type]'=> 'eq',
            'searchCriteria[filter_groups][1][filters][0][field]'=> 'created_at',
            'searchCriteria[filter_groups][1][filters][0][value]'=> $date,
            'searchCriteria[filter_groups][1][filters][0][condition_type]'=> 'gteq',
            'searchCriteria[pageSize]'=> 1);
        $url = $url.'?'.http_build_query($params);
        $token='Bearer '.$magentoAcessToken;
        $this->curlClient->addHeader("Authorization", $token);
        try{
        $this->curlClient->get($url);}
        catch (\Exception $e) {
            $errorMessage='Invalid Magento Base Url or Basic Authorization token. Please try again. If the issue still persists, please contact enterprise@quivers.com for further assistance.';
            $blockObj->getBackendSession()->unsMyValue();
            $blockObj->getBackendSession()->setMyName('InActive');
            throw new CouldNotSaveException(
                __($errorMessage));
        }
        $code=$this->curlClient->getStatus();
        $response=$this->curlClient->getBody();
        if($code>=400){
        $blockObj->getBackendSession()->unsMyValue();
        $blockObj->getBackendSession()->setMyName('InActive');
        if($code == 404){
            $blockObj->getCatalogSession()->setMyName('quivers_parameters_magento_base_url');
            $errorMessage='Invalid Magento Base Url. Please try again. If the issue still persists, please contact enterprise@quivers.com for further assistance.';
            }
        elseif($code == 401){
            $blockObj->getCatalogSession()->setMyName('quivers_parameters_magento_access_token');
            $errorMessage='Invalid Basic Authorization. Please try again. If the issue still persists, please contact enterprise@quivers.com for further assistance.';
            }
        else {
            $errorMessage='Invalid Magento Base Url or Basic Authorization token. Please try again. If the issue still persists, please contact enterprise@quivers.com for further assistance.';}
        throw new CouldNotSaveException(
                __($errorMessage));
            }

        }
}
