/**
 * Copyright © Magento, Inc. All rights reserved.
 * See COPYING.txt for license details.
 */

/**
 * @api
 */

define([
    'Quivers_Integration/js/view/checkout/summary/tax',
    'Magento_Checkout/js/model/totals'
], function (Component, totals) {
    'use strict';

    var isFullTaxSummaryDisplayed = window.checkoutConfig.isFullTaxSummaryDisplayed,
        isActive = window.checkoutConfig.validation,
        isZeroTaxDisplayed = window.checkoutConfig.isZeroTaxDisplayed;

    return Component.extend({
        /**
         * @override
         */
        ifShowValue: function () {
            if (this.getPureValue() === 0) {
                return isZeroTaxDisplayed;
            }

            return true;
        },

        ifShowAddTax : function(){
            return this.getPureValue() > 0;
        },

        ifActiveQuivers : function(){
            return isActive;
        },

        /**
         * @override
         */
        ifShowDetails: function () {
            return this.getPureValue() > 0 && isFullTaxSummaryDisplayed;
        },

          /**
         * @return {Boolean}
         */
        ifShowIncludedTax: function () {
            return this.getPureIncludedTax() > 0;
        }, 

        /**
         * @return {Number}
         */       
        getIncludedTax: function () {
            var includedTaxTotal = totals.getSegment('included_tax');

            var includedTax = 0;
            
            if (includedTaxTotal) {
                includedTax = includedTaxTotal.value;
            }
            return this.getFormattedPrice(includedTax);
        },

        /**
         * @return {*|String}
         */
        getPureIncludedTax : function () {
            var includedTaxTotal = totals.getSegment('included_tax');

            var includedTax = 0;
            
            if (includedTaxTotal) {
                includedTax = includedTaxTotal.value;
            }
            return includedTax;
        },

        /**
         * @override
         */
        isCalculated: function () {
            return this.totals() && totals.getSegment('tax') !== null;
        }
    });
});

